/******************************************************************************
 *
 * Licensed Materials - Property of IBM
 *
 * (C) COPYRIGHT IBM Corp. 2006
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *
 *****************************************************************************/

package examples.api.recon;

import java.rmi.*;
import java.util.*;
import java.io.*;
import com.ibm.itim.apps.recon.*;
import com.ibm.itim.apps.*;
import com.ibm.itim.apps.exception.AppProcessingException;
import com.ibm.itim.apps.jaas.callback.*;
import com.ibm.itim.apps.provisioning.*;
import com.ibm.itim.apps.search.*;
import com.ibm.itim.dataservices.model.*;
import com.ibm.itim.dataservices.model.domain.*;
import com.ibm.itim.remoteservices.exception.RemoteServicesException;
import examples.api.recon.callback.ReconCompleteCallback;

import javax.security.auth.*;
import javax.security.auth.login.*;

/**
 * Sample command-line Java class to add recon.unit for a service.
**/
public class AddReconUnit {

   private static final String LOGIN_CONTEXT   = "ITIM";

   /**
    * Command line argument names (prefixed by "-")
    */

   private static final String SERVICE_PROFILE    = "serviceprofile";
   private static final String SERVICE_FILTER     = "servicefilter";
   private static final String RECON_FILTER       = "reconfilter";
   private static final String EXCLUDE_ATTRIBUTES = "excludeattributes";
   private static final String MONTH              = "month";
   private static final String DAY                = "day";
   private static final String DAY_OF_WEEK        = "dayofweek";
   private static final String HOUR               = "hour";
   private static final String MINUTE             = "minute";
   private static final String MAX_DURATION       = "maxduration";
   private static final String LOCK_SERVICE       = "lockservice";
   private static final String RECON_CALLBACK     = "reconcallback";

   /**
    * main method.
    */
   public static void main(String[] args) {

       Hashtable arguments = parseArgs(args);

       try {
            String contextFactory = System.getProperty("apps.context.factory");
            String appServerUrl   = System.getProperty("apps.server.url");
            String ejbUser        = System.getProperty("apps.ejb.user");
            String ejbPswd        = System.getProperty("apps.ejb.pswd");
            String itimUser       = System.getProperty("itim.user");
            String itimPswd       = System.getProperty("itim.pswd");
            String tenantId       = System.getProperty("tenant.id");
            String ldapServerRoot = System.getProperty("ldapserver.root");

            Hashtable env = new Hashtable();
            env.put(InitialPlatformContext.CONTEXT_FACTORY, contextFactory);
            env.put(PlatformContext.PLATFORM_URL, appServerUrl);
            env.put(PlatformContext.PLATFORM_PRINCIPAL, ejbUser);
            env.put(PlatformContext.PLATFORM_CREDENTIALS, ejbPswd);

            System.out.println("\nCreating new PlatformContext \n");
            PlatformContext platform = new InitialPlatformContext(env);
            PlatformCallbackHandler handler = new PlatformCallbackHandler(
                                                      itimUser, itimPswd);
            handler.setPlatformContext(platform);

            System.out.println("Logging in \n");
            // Associate the CallbackHandler with a LoginContext,
            LoginContext lc = new LoginContext(LOGIN_CONTEXT, handler);
            // Try to authenticate the user with the platform
            lc.login();

            System.out.println("Getting subject \n");
            // Extract the authenticated JAAS Subject from the LoginContext
            Subject subject = lc.getSubject();


            System.out.println("Searching for Service \n");
            // Use the Search API to locate the the Service instance
            SearchMO searchMO = new SearchMO(platform, subject);
            searchMO.setContext(new CompoundDN(new 
                      DistinguishedName("ou="+tenantId+"," + ldapServerRoot)));
            searchMO.setCategory(ObjectProfileCategory.SERVICE);
            searchMO.setFilter((String)arguments.get(SERVICE_FILTER));
            searchMO.setProfileName((String)arguments.get(SERVICE_PROFILE));
            Collection services = searchMO.execute().getResults();
            if(services == null || services.isEmpty()) {

                System.out.println("Unable to find Service.\n");
                System.exit(0);
            }

            Service service = (Service)services.iterator().next();
            ServiceMO serviceMO =
               new ServiceMO(platform, subject, service.getDistinguishedName());

            System.out.println("Creating ReconManager \n");
            ReconManager reconManager = new ReconManager(platform, subject);


            // Initialize temporary reconciliation unit
            ReconUnitData newReconUnitData = new ReconUnitData();

            // Set month
            int month = Integer.parseInt((String)arguments.get(MONTH));
            if(month != -1 && !(month > 0 && month < 13)) {
                System.out.println("Invalid month specified. " + 
                    "Valid values : 1-12 and -1 stands for Monthly recon.\n");
                System.exit(0);
            }
            newReconUnitData.setMonth(month);

            // Set day of month
            if((String)arguments.get(DAY) != null) {
                int day = Integer.parseInt((String)arguments.get(DAY));
                if(day != -1 && !(day > 0 && day < 32)) {
                    System.out.println("Invalid day specified. " + 
                        "Valid values : 1-31 and -1 stands for Daily recon.\n");
                    System.exit(0);
                }
                newReconUnitData.setDayOfMonth(day);
            }

            //Set day of week
            //Should be set only if the day of month is not required
            if((String)arguments.get(DAY) == null && (String)arguments.get(DAY_OF_WEEK) != null) {
                int dayOfWeek = Integer.parseInt((String)arguments.get(DAY_OF_WEEK));
                if(dayOfWeek < 1 || dayOfWeek > 7) {
                    System.out.println("Invalid day of week specified. " + 
                        "Valid values 1-7.(1 = Sunday, 2 = Monday, ...) \n");
                    System.exit(0);
                }
                newReconUnitData.setDayOfWeek(dayOfWeek);
            }

            // Set hour
            int hour  = Integer.parseInt((String)arguments.get(HOUR));
            if(hour < -1 || hour > 23) {
                System.out.println("Invalid hour specified. " + 
                    "Valid values : 0-23 and -1 stands for Hourly recon.\n");
                System.exit(0);
            }            
            newReconUnitData.setHour(hour);

            // Set minute
            int minute = Integer.parseInt((String)arguments.get(MINUTE));
            if(minute < 0 || minute > 59) {
                System.out.println("Invalid minute specified. " + 
                    "Valid values : 0-59.\n");
                System.exit(0);
            }
            newReconUnitData.setMinute(minute);

            // Set max duration of reconciliation
            String strMaxDuration = (String)arguments.get(MAX_DURATION);
            if(strMaxDuration != null && strMaxDuration.trim().length() > 0) {
                newReconUnitData.setMaxDuration(
                            Long.valueOf(strMaxDuration).longValue());
            }

            // Set lock-service setting
            String lockService = (String)arguments.get(LOCK_SERVICE);
            if(lockService != null) {
                newReconUnitData.setLockService(
                            Boolean.valueOf(lockService).booleanValue());
            }

            // Set reconciliation filter if supported
            String reconQueryFilter  = (String) arguments.get(RECON_FILTER);
            if(reconManager.isFilterSupported(serviceMO)) {
                newReconUnitData.setReconQueryFilter(reconQueryFilter);
            } else {
                System.out.println("Recon query filter not supported, query is ignored.\n");
            }

            // Set reconciliation attributes to be included.
            String excludeAttributes = (String) arguments.get(EXCLUDE_ATTRIBUTES);
            if(excludeAttributes != null) {

                Collection attributes = reconManager.
                              getAvailableReconciliationAttributes(serviceMO);
                String[] excludeAttribList = excludeAttributes.split(",");
                ArrayList excludeQueryAttribures = new ArrayList();
                for(int i=0;i<excludeAttribList.length; i++) {
                   excludeQueryAttribures.add(excludeAttribList[i]);
                }
                newReconUnitData.setReconQueryAttributes(attributes, 
                                          excludeQueryAttribures);
            }

            System.out.println("Adding the recon unit..\n");
            String reconcallback = (String) arguments.get (RECON_CALLBACK);
            if(Boolean.valueOf(reconcallback).booleanValue()) {
                reconManager.addReconUnitData(serviceMO, newReconUnitData, new ReconCompleteCallback());
            } else {

                reconManager.addReconUnitData(serviceMO, newReconUnitData, null);
                /*
                This is an alternative way of adding\changing\removing
                more than one reconciliation units in one function call -
                    reconManager.setReconUnits, 
                without callback information.
                All the existing set of reconciliation units associated
                with the resource will be replaced by the new 
                list passed to 'setReconUnits'.

                Collection reconUnits = reconManager.getReconUnits(serviceMO);
                if(reconUnits == null) {
                    reconUnits = new ArrayList();
                }
                reconUnits.add(newReconUnitData1);
                reconUnits.add(newReconUnitData2);
                reconUnits.add(newReconUnitData3);

                System.out.println("Setting the recon units..\n");
                reconManager.setReconUnits(serviceMO, reconUnits);
                System.out.println("\nRecon units set successfully.");
                */
            }

            System.out.println("\nAdded recon unit successfully.");

       } catch (RemoteException e) {
           e.printStackTrace();
       } catch (LoginException e) {
           e.printStackTrace();
       } catch (SchemaViolationException e) {
           e.printStackTrace();
       } catch (AuthorizationException e) {
           e.printStackTrace();
       } catch (ApplicationException e) {
           e.printStackTrace();
       } catch (NumberFormatException e) {
           System.out.println("\nInvalid number format, enter valid numbers\n");
       } catch (AppProcessingException e) {
           e.printStackTrace();
       } catch (RemoteServicesException e) {
           e.printStackTrace();    
       } catch (IOException e) {
           System.out.println("File not Found\n");
       }
   }

   /**
    * Parses the argument list from the command-line
    */
   public static Hashtable parseArgs(String[] args) {
       Hashtable arguments = new Hashtable();
       String argumentList = "";
       for (int i=0;i<args.length;i++) {
           argumentList += args[i];
       }
       try {
           StringTokenizer tokenizer =
               new StringTokenizer(argumentList, "-");
           while (tokenizer.hasMoreTokens()) {
               String token = (String)tokenizer.nextToken();
               int delim = token.indexOf("?");
               String name = token.substring(0, delim);
               String value = token.substring(delim+1, token.length());
               if (arguments.get(name) != null) {
                   // arg name used previous
                   Object vals = arguments.get(name);
                   if (vals instanceof String) {
                       // convert to String[]
                       Vector values = new Vector(1);
                       values.add((String)vals);
                       values.add(value);
                       arguments.put(name, values);
                   } else if (vals instanceof Vector) {
                       // add new element to String[]
                       Vector values = (Vector)vals;
                       values.add(value);
                       arguments.put(name, vals);
                   }
               } else {
                   arguments.put(name, value);
               }
           }
           if (!checkArguments(arguments))
               System.exit(0);
       } catch (Exception e) {
           e.printStackTrace();
           printUsage("Invalid command syntax");
           System.exit(0);
       }
       return arguments;
   }

   public static boolean checkArguments(Hashtable arguments) {
       if (!arguments.containsKey(SERVICE_PROFILE))
           return printUsage("No Service profile specified");
       if (!arguments.containsKey(SERVICE_FILTER))
           return printUsage("No service filter specified");
       if (!arguments.containsKey(MONTH))
           return printUsage("Month not specified");
       if (!(arguments.containsKey(DAY) || arguments.containsKey(DAY_OF_WEEK)))
           return printUsage("Day or Day of week not specified");
       if (!arguments.containsKey(HOUR))
           return printUsage("Hour not specified");
       if (!arguments.containsKey(MINUTE))
           return printUsage("Minute not specified");
       return true;
   }

   public static boolean printUsage(String msg) {
       StringBuffer usage = new StringBuffer();
       usage.append("\naddReconUnit: " + msg + "\n");
       usage.append("\nusage: addReconUnit -[argument] ? \"[value]\"\n");
       usage.append("\n");
       usage.append("-serviceprofile      Profile Name of the Service");
       usage.append(" (e.g., WinLocalProfile)\n");
       usage.append("-servicefilter       Ldap Filter to search for a Service instance\n");
       usage.append("-month               month of recon. Valid values 1-12, -1 indicates Monthly\n");
       usage.append("-day                 day of recon. Valid values 1-31, -1 indicates Daily.\n");
       usage.append("-dayofweek           day of week. Valid values 1-7.(1 = Sunday, 2 = Monday,..)\n");
       usage.append("\nNOTE: Either day or dayofweek is mandatory. \n");
       usage.append("If specified both, then dayofweek will be ignored\n\n");
       usage.append("-hour                hour of recon. Valid values 0-23, -1 indicates Hourly.\n");
       usage.append("-minute              minute of recon. Valid values 0-59.\n\n");
       usage.append("-[reconfiler]        Ldap filter to reconcile accounts.\n");
       usage.append("-[excludeattributes] Attribute(s) separated by comma");
       usage.append(" to be excluded during \n                     reconciliation.\n");
       usage.append("-[lockservice]       Specifies if the Service needs to be locked during recon,\n");
       usage.append("                     default value = \"true\"\n");
       usage.append("-[maxduration]       Maximum duration(in minutes) for recon process to run.\n");
       usage.append("                     default value 600 minutes.\n");
       usage.append("-[reconcallback]     If the recon completion details needs to be logged in \n");
       usage.append("                     SystemOut.log. Default value \"false\"\n\n");
       usage.append("Arguments enclosed in [] are optional\n\n");
       usage.append("Example: addReconUnit -serviceprofile?WinLocalProfile  -servicefilter?");
       usage.append("\"(erservicename=WinLocal)\" -reconfilter?\"(eruid=a*)\"");
       usage.append(" -excludeattributes?\"erntpasswordneverexpires,description\"");
       usage.append(" -maxduration?10 -month?10 -day?26 -hour?10 -minute?10 -reconcallback?\"true\"");
       usage.append("\n\nNOTE: If negative value -1 needs to be entered for month,day or hour, then \n");
       usage.append("change the StringTokenizer to '+' instead of '-' in function \n");
       usage.append("AddReconUnit.parseArgs() to identify the arguments prefixed with '+'");
       System.out.println(usage);
       return false;
   }

}
